from osgeo import gdal
import numpy as np
import os 
import warnings
warnings.filterwarnings('ignore')

from .basic_func import read_bin, write_bin, conv2d

def dopfp(T3_folder,window_size=1,dop_type=1,write_flag=None):
	T11 = read_bin(T3_folder+"/T11.bin")
	T22 = read_bin(T3_folder+"/T22.bin")
	T33 = read_bin(T3_folder+"/T33.bin")

	T12_i = read_bin(T3_folder+'/T12_imag.bin')
	T12_r = read_bin(T3_folder+'/T12_real.bin')
	T13_i = read_bin(T3_folder+'/T13_imag.bin')
	T13_r = read_bin(T3_folder+'/T13_real.bin')
	T23_i = read_bin(T3_folder+'/T23_imag.bin')
	T23_r = read_bin(T3_folder+'/T23_real.bin')

	T12 = T12_r + 1j*T12_i
	T13 = T13_r + 1j*T13_i
	T23 = T23_r + 1j*T23_i
	    
	t11_T1 = T11
	t12_T1 = T12
	t13_T1 = T13
	t21_T1 = np.conj(T12)
	t22_T1 = T22
	t23_T1 = T23
	t31_T1 = np.conj(T13)
	t32_T1 = np.conj(T23)
	t33_T1 = T33
	del T11, T22, T33,T12, T13, T23, T12_i,T12_r, T13_i, T13_r, T23_i, T23_r
	            
	kernel = np.ones((window_size,window_size),np.float32)/(window_size*window_size)

	        
	t11_T1r = conv2d(np.real(t11_T1),kernel)
	t11_T1i = conv2d(np.imag(t11_T1),kernel)
	t11s = t11_T1r+1j*t11_T1i

	t12_T1r = conv2d(np.real(t12_T1),kernel)
	t12_T1i = conv2d(np.imag(t12_T1),kernel)
	t12s = t12_T1r+1j*t12_T1i

	t13_T1r = conv2d(np.real(t13_T1),kernel)
	t13_T1i = conv2d(np.imag(t13_T1),kernel)
	t13s = t13_T1r+1j*t13_T1i

	t21_T1r = conv2d(np.real(t21_T1),kernel)
	t21_T1i = conv2d(np.imag(t21_T1),kernel)
	t21s = t21_T1r+1j*t21_T1i

	t22_T1r = conv2d(np.real(t22_T1),kernel)
	t22_T1i = conv2d(np.imag(t22_T1),kernel)
	t22s = t22_T1r+1j*t22_T1i

	t23_T1r = conv2d(np.real(t23_T1),kernel)
	t23_T1i = conv2d(np.imag(t23_T1),kernel)
	t23s = t23_T1r+1j*t23_T1i
	            
	t31_T1r = conv2d(np.real(t31_T1),kernel)
	t31_T1i = conv2d(np.imag(t31_T1),kernel)
	t31s = t31_T1r+1j*t31_T1i

	t32_T1r = conv2d(np.real(t32_T1),kernel)
	t32_T1i = conv2d(np.imag(t32_T1),kernel)
	t32s = t32_T1r+1j*t32_T1i

	t33_T1r = conv2d(np.real(t33_T1),kernel)
	t33_T1i = conv2d(np.imag(t33_T1),kernel)
	t33s = t33_T1r+1j*t33_T1i

	if dop_type==0:
		dop_fp=0
	# Barakat DOP
	if dop_type==1:            
	    det_T3 = t11s*(t22s*t33s-t23s*t32s)-t12s*(t21s*t33s-t23s*t31s)+t13s*(t21s*t32s-t22s*t31s)
	    trace_T3 = t11s + t22s + t33s
	    dop_fp = np.sqrt(1-((27*det_T3)/trace_T3**3))

	if write_flag:
	    infile = T3_folder+'/T11.bin'
	    """Write files to disk"""
	    if os.path.exists(T3_folder+'/T11.bin'):
	        infile = iFolder+'/T11.bin'
	    elif os.path.exists(T3_folder+'/C11.bin'):
	        infile = T3_folder+'/C11.bin'

	    ofilegrvi = T3_folder+'/dop_fp.bin'
	    write_bin(ofilegrvi,dop_fp,infile)

	return dop_fp



from osgeo import gdal
import numpy as np
import os
import warnings
warnings.filterwarnings('ignore')

from .basic_func import read_bin, write_bin, conv2d

def mf4cf(T3_folder,window_size=1,write_flag=None):
        
    T11 = read_bin(T3_folder+"/T11.bin").astype(np.float32)
    T22 = read_bin(T3_folder+"/T22.bin").astype(np.float32)
    T33 = read_bin(T3_folder+"/T33.bin").astype(np.float32)

    T12_i = read_bin(T3_folder+'/T12_imag.bin').astype(np.float32)
    T12_r = read_bin(T3_folder+'/T12_real.bin').astype(np.float32)
    T13_i = read_bin(T3_folder+'/T13_imag.bin').astype(np.float32)
    T13_r = read_bin(T3_folder+'/T13_real.bin').astype(np.float32)
    T23_i = read_bin(T3_folder+'/T23_imag.bin').astype(np.float32)
    T23_r = read_bin(T3_folder+'/T23_real.bin').astype(np.float32)

    T12 = T12_r + 1j*T12_i
    T13 = T13_r + 1j*T13_i
    T23 = T23_r + 1j*T23_i
        
    t11_T1 = T11
    t12_T1 = T12
    t13_T1 = T13
    t21_T1 = np.conj(T12)
    t22_T1 = T22
    t23_T1 = T23
    t31_T1 = np.conj(T13)
    t32_T1 = np.conj(T23)
    t33_T1 = T33

    del T11, T22, T33,T12, T13, T23, T12_i,T12_r, T13_i, T13_r, T23_i, T23_r

    kernel = np.ones((window_size,window_size),np.float32)/(window_size*window_size)


    t11_T1r = conv2d(np.real(t11_T1),kernel)
    t11_T1i = conv2d(np.imag(t11_T1),kernel)
    t11s = t11_T1r+1j*t11_T1i

    t12_T1r = conv2d(np.real(t12_T1),kernel)
    t12_T1i = conv2d(np.imag(t12_T1),kernel)
    t12s = t12_T1r+1j*t12_T1i

    t13_T1r = conv2d(np.real(t13_T1),kernel)
    t13_T1i = conv2d(np.imag(t13_T1),kernel)
    t13s = t13_T1r+1j*t13_T1i


    t21_T1r = conv2d(np.real(t21_T1),kernel)
    t21_T1i = conv2d(np.imag(t21_T1),kernel)
    t21s = t21_T1r+1j*t21_T1i

    t22_T1r = conv2d(np.real(t22_T1),kernel)
    t22_T1i = conv2d(np.imag(t22_T1),kernel)
    t22s = t22_T1r+1j*t22_T1i

    t23_T1r = conv2d(np.real(t23_T1),kernel)
    t23_T1i = conv2d(np.imag(t23_T1),kernel)
    t23s = t23_T1r+1j*t23_T1i
    

    
    t31_T1r = conv2d(np.real(t31_T1),kernel)
    t31_T1i = conv2d(np.imag(t31_T1),kernel)
    t31s = t31_T1r+1j*t31_T1i

    t32_T1r = conv2d(np.real(t32_T1),kernel)
    t32_T1i = conv2d(np.imag(t32_T1),kernel)
    t32s = t32_T1r+1j*t32_T1i

    t33_T1r = conv2d(np.real(t33_T1),kernel)
    t33_T1i = conv2d(np.imag(t33_T1),kernel)
    t33s = t33_T1r+1j*t33_T1i

    det_T3 = t11s*(t22s*t33s-t23s*t32s)-t12s*(t21s*t33s-t23s*t31s)+t13s*(t21s*t32s-t22s*t31s)
    trace_T3 = t11s + t22s + t33s
    m1 = np.real(np.sqrt(1-(27*(det_T3/(trace_T3**3)))))

    k11_f = (t11s + t22s + t33s)/2
    k44_f = (-t11s + t22s + t33s)/2
    k14_f = np.imag(t23s)

    trace_T3 = t11s + t22s + t33s
 

    s0_f = trace_T3
    dop_f = m1

    val1 = (4*dop_f*k11_f*k44_f)/(k44_f**2 - (1 + 4*dop_f**2)*k11_f**2)
    val2 = np.abs(k14_f)/(k11_f)
    


    theta_f = np.real(np.arctan(val1)) # separation for surface and dbl
    tau_f = np.real(np.arctan(val2)) # separation for helix
    # thet = np.rad2deg(thet)
    theta_FP = np.rad2deg(theta_f).astype(np.float32)
    tau_FP = np.rad2deg(tau_f).astype(np.float32)

    

    pc_f = (dop_f*s0_f*(np.sin(2*tau_f))).astype(np.float32)
    pv_f = ((1-dop_f)*s0_f).astype(np.float32)
    res_pow = s0_f - (pc_f + pv_f)
    ps_f = ((res_pow/2)*(1+np.sin((2*theta_f)))).astype(np.float32)
    pd_f = ((res_pow/2)*(1-np.sin((2*theta_f)))).astype(np.float32)
    
    if write_flag:
        infile = T3_folder+'/T11.bin'
        """Write files to disk"""
        if os.path.exists(T3_folder+'/T11.bin'):
            infile = iFolder+'/T11.bin'
        elif os.path.exists(T3_folder+'/C11.bin'):
            infile = T3_folder+'/C11.bin'

        ofilegrvi0 = T3_folder+'/Tau_FP_4c.bin'
        write_bin(ofilegrvi0,tau_FP,infile)

        ofilegrvi = T3_folder+'/Theta_FP_4c.bin'
        write_bin(ofilegrvi,theta_FP,infile)
        
        ofilegrvi1 = T3_folder+'/Pd_FP_4c.bin'
        write_bin(ofilegrvi1,pd_f,infile)
        
        ofilegrvi2 = T3_folder+'/Ps_FP_4c.bin'
        write_bin(ofilegrvi2,ps_f,infile)
        
        ofilegrvi3 = T3_folder+'/Pv_FP_4c.bin'
        write_bin(ofilegrvi3,pv_f,infile)

        ofilegrvi4 = T3_folder+'/Pc_FP_4c.bin'
        write_bin(ofilegrvi4,pc_f,infile)     

    return ps_f, pd_f, pv_f,pc_f,theta_FP,tau_FP


from osgeo import gdal
import numpy as np
import os 
import warnings
warnings.filterwarnings('ignore')

from .basic_func import read_bin, write_bin, conv2d


def grvi(T3_folder,window_size=1,write_flag=None):

    T11 = read_bin(T3_folder+"/T11.bin")
    T22 = read_bin(T3_folder+"/T22.bin")
    T33 = read_bin(T3_folder+"/T33.bin")

    T12_i = read_bin(T3_folder+'/T12_imag.bin')
    T12_r = read_bin(T3_folder+'/T12_real.bin')
    T13_i = read_bin(T3_folder+'/T13_imag.bin')
    T13_r = read_bin(T3_folder+'/T13_real.bin')
    T23_i = read_bin(T3_folder+'/T23_imag.bin')
    T23_r = read_bin(T3_folder+'/T23_real.bin')
    T12 = T12_r + 1j*T12_i
    T13 = T13_r + 1j*T13_i
    T23 = T23_r + 1j*T23_i

    T3 = np.dstack((T11,T12,T13,np.conj(T12),T22,T23,np.conj(T13),np.conj(T23),T33))
   
    DOP = np.zeros([np.size(T3,0),np.size(T3,1)])
    prvi = np.zeros([np.size(T3,0),np.size(T3,1)])
    "Special Unitary Matrix"
    D = (1/np.sqrt(2))*np.array([[1,0,1], [1,0,-1],[0,np.sqrt(2),0]])
    
    for i in range(np.size(T3,0)):

        self.pBar.emit((i/np.size(T3,0))*100)
        for j in range(np.size(T3,1)):
            det = np.abs(np.linalg.det(T3[i,j,:].reshape((3, 3))))
            trace = np.abs(np.trace(T3[i,j,:].reshape((3, 3))))
            if trace==0:
                DOP[i][j]=0
            # elif((27*det/trace**3)>1.0):
            #     DOP[i][j]=0
            else:
                DOP[i][j] = np.sqrt(1-((27*det)/trace**3))
            
            tempT3 =  np.reshape(T3[i,j,:],(3,3))
            C3 = np.matmul(np.matmul((D.T),tempT3),D).flatten()
            
            prvi[i][j] =(1- DOP[i][j])*C3[4]*0.5 # (1-dop)*vh


    if write_flag:
        infile = T3_folder+'/T11.bin'
        """Write files to disk"""
        if os.path.exists(T3_folder+'/T11.bin'):
            infile = iFolder+'/T11.bin'
        elif os.path.exists(T3_folder+'/C11.bin'):
            infile = T3_folder+'/C11.bin'

        ofilegrvi = T3_folder+'/PRVI.bin'
        write_bin(ofilegrvi,prvi,infile)
                    

    return prvi



from osgeo import gdal
import numpy as np
import os 
# import warnings
from shutil import copyfile
# warnings.filterwarnings('ignore')

from .basic_func import read_bin, write_bin, conv2d

def T3_C3(T3_stack):
    nrows = np.size(T3_stack,0)
    ncols = np.size(T3_stack,1)
    
    D = (1/np.sqrt(2))*np.array([[1,0,1], [1,0,-1],[0,np.sqrt(2),0]])
    TT1 = np.dstack((T3_stack[:,:,0].flatten(), T3_stack[:,:,1].flatten(), T3_stack[:,:,2].flatten(),
                     T3_stack[:,:,3].flatten(), T3_stack[:,:,4].flatten(), T3_stack[:,:,5].flatten(),
                     T3_stack[:,:,6].flatten(), T3_stack[:,:,7].flatten(), T3_stack[:,:,8].flatten()))
    TT1 = TT1[0,:,:]
    TT1 = np.reshape(TT1,(TT1.shape[0],3,3))
    TT1 = np.matmul(np.matmul((D.T),TT1),D)
    TT1 = np.reshape(TT1,(TT1.shape[0],9))

    return np.reshape(TT1,(nrows,ncols,9)) 


def C3_compact_C2(C3,psi,chi):


	C11 = C3[:,:,0]
	C12 = C3[:,:,1]
	C13 = C3[:,:,2]

	C21 = C3[:,:,3]
	C22 = C3[:,:,4]
	C23 = C3[:,:,5]

	C31 = C3[:,:,6]
	C32 = C3[:,:,7]
	C33 = C3[:,:,8]


	#%%
	psi = psi*np.pi/180
	chi = chi*np.pi/180
	 
	CP11 = 0.5*((1+np.cos(2*psi)*np.cos(2*chi))*C11+
	             0.5*(1-np.cos(2*psi)*np.cos(2*chi))*C22+
	             (1/np.sqrt(2))*(np.sin(2*psi)*np.cos(2*chi))*(C12+np.conj(C12))+
	                (1j/np.sqrt(2))*np.sin(2*chi)*(C12-np.conj(C12)) 
	           )

	CP12 = 0.5*((1/np.sqrt(2))*(1+np.cos(2*psi)*np.cos(2*chi))*C3[:,:,1]+
	            (1/np.sqrt(2))*(1-np.cos(2*psi)*np.cos(2*chi))*C3[:,:,5]+
	            (np.sin(2*psi)*np.cos(2*chi))*(C3[:,:,2]+0.5*C3[:,:,4])+
	            1j*np.sin(2*chi)*(C3[:,:,2]-0.5*C3[:,:,4])
	            )

	CP22 = 0.5*(0.5*(1+np.cos(2*psi)*np.cos(2*chi))*C3[:,:,4]+
	                (1-np.cos(2*psi)*np.cos(2*chi))*C3[:,:,8]+
	              (1/np.sqrt(2))*(np.sin(2*psi)*np.cos(2*chi))*(C3[:,:,5]+np.conj(C3[:,:,5]))+
	                (1j/np.sqrt(2))*np.sin(2*chi)*(C3[:,:,5]-np.conj(C3[:,:,5])) 
	            )

	return CP11, CP12, CP22

def C3_c2(inFolder):

	if os.path.isfile(inFolder+'/C11.bin'):
	    
	    C3_stack = load_C3(inFolder)
	    # T3_stack = C3_T3(C3_stack)
	    
	    if os.path.isdir(inFolder+'\\C2_pp1'):
	        pass
	    else:
	        os.mkdir(inFolder+'\\C2_pp1')
	        
	    infile = inFolder+'/C11.bin'
	    out_file =  inFolder+'\\C2_pp1'+'/C11.bin'
	    write_bin(out_file,np.real(C3_stack[:,:,0]),infile)
	    # infile = inFolder+'/C11.bin'
	    out_file =  inFolder+'\\C2_pp1'+'/C12_real.bin'
	    write_bin(out_file,np.real((C3_stack[:,:,1])/np.sqrt(2)),infile)
	    out_file =  inFolder+'\\C2_pp1'+'/C12_imag.bin'
	    write_bin(out_file,np.imag((C3_stack[:,:,1])/np.sqrt(2)),infile)
	    out_file =  inFolder+'\\C2_pp1'+'/C22.bin'
	    write_bin(out_file,np.real((C3_stack[:,:,4])/(2)),infile)    
	    
	    # copy config file
	    copyfile(inFolder+'/config.txt', inFolder+'\\C2_pp1\\config.txt')
	    
	    print('completed'+inFolder+'\\C2_pp1')
	    # ################## PP2 ###############
	    if os.path.isdir(inFolder+'\\C2_pp2'):
	        pass
	    else:
	        os.mkdir(inFolder+'\\C2_pp2')
	        
	    # infile = inFolder+'/C11.bin'
	    out_file =  inFolder+'\\C2_pp2'+'/C11.bin'
	    write_bin(out_file,np.real(C3_stack[:,:,8]),infile)
	    # infile = inFolder+'/C11.bin'
	    out_file =  inFolder+'\\C2_pp2'+'/C12_real.bin'
	    write_bin(out_file,np.real((C3_stack[:,:,5])/np.sqrt(2)),infile)
	    out_file =  inFolder+'\\C2_pp2'+'/C12_imag.bin'
	    write_bin(out_file,np.imag((C3_stack[:,:,5])/np.sqrt(2)),infile)
	    out_file =  inFolder+'\\C2_pp2'+'/C22.bin'
	    write_bin(out_file,np.real((C3_stack[:,:,4])/(2)),infile)  
	    
	    # copy config file
	    copyfile(inFolder+'/config.txt', inFolder+'\\C2_pp2\\config.txt')
	    
	    print('completed'+inFolder+'\\C2_pp2')
	    ################## PP3 ###############
	    if os.path.isdir(inFolder+'\\C2_pp3'):
	        pass
	    else:
	        os.mkdir(inFolder+'\\C2_pp3')
	        
	    # infile = inFolder+'/C11.bin'
	    out_file =  inFolder+'\\C2_pp3'+'/C11.bin'
	    write_bin(out_file,np.real(C3_stack[:,:,0]),infile)
	    # infile = inFolder+'/C11.bin'
	    out_file =  inFolder+'\\C2_pp3'+'/C12_real.bin'
	    write_bin(out_file,np.real(C3_stack[:,:,2]),infile)
	    out_file =  inFolder+'\\C2_pp3'+'/C12_imag.bin'
	    write_bin(out_file,np.imag(C3_stack[:,:,2]),infile)
	    out_file =  inFolder+'\\C2_pp3'+'/C22.bin'
	    write_bin(out_file,np.real((C3_stack[:,:,8])),infile)  
	    
	    # copy config file
	    copyfile(inFolder+'/config.txt', inFolder+'\\C2_pp3\\config.txt')
	else:
		pass

def S2_T3(S11,S12,S22):

	# Kp- 3-D Pauli feature vector
	Kp = np.array([S11+S22, S11-S22, S12])

	# 3x3 Pauli Coherency Matrix elements

	T11 = np.abs(Kp[0])**2
	T22 = np.abs(Kp[1])**2
	T33 = np.abs(Kp[2])**2

	T12 = Kp[0]*np.conj(Kp[1])
	T13 = Kp[0]*np.conj(Kp[2])
	T23 = Kp[1]*np.conj(Kp[2])
	return np.dstack((T11,T12,T13,np.conjugate(T12),T22,T23,np.conjugate(T13),T23,T33))



from osgeo import gdal
import numpy as np
import os 
import warnings
warnings.filterwarnings('ignore')

from .basic_func import read_bin, write_bin, conv2d


def grvi(T3_folder,window_size=1,write_flag=None):

    T11 = read_bin(T3_folder+"/T11.bin")
    T22 = read_bin(T3_folder+"/T22.bin")
    T33 = read_bin(T3_folder+"/T33.bin")

    T12_i = read_bin(T3_folder+'/T12_imag.bin')
    T12_r = read_bin(T3_folder+'/T12_real.bin')
    T13_i = read_bin(T3_folder+'/T13_imag.bin')
    T13_r = read_bin(T3_folder+'/T13_real.bin')
    T23_i = read_bin(T3_folder+'/T23_imag.bin')
    T23_r = read_bin(T3_folder+'/T23_real.bin')

    T12 = T12_r + 1j*T12_i
    T13 = T13_r + 1j*T13_i
    T23 = T23_r + 1j*T23_i
        
    t11_T1 = T11
    t12_T1 = T12
    t13_T1 = T13
    t21_T1 = np.conj(T12)
    t22_T1 = T22
    t23_T1 = T23
    t31_T1 = np.conj(T13)
    t32_T1 = np.conj(T23)
    t33_T1 = T33
    del T11, T22, T33,T12, T13, T23, T12_i,T12_r, T13_i, T13_r, T23_i, T23_r
                
    nrows  = np.shape(T11)[1]
    ncols = np.shape(T11)[0]
    # nrows  = 100
    # ncols = 100
   
    temp_rvi = np.zeros((ncols,nrows))
                
    # %% for window processing
    wsi=wsj=ws
    
    inci=int(np.fix(wsi/2)) # Up & down movement margin from the central row
    incj=int(np.fix(wsj/2)) # Left & right movement from the central column
    # % Starting row and column fixed by the size of the patch extracted from the image of 21/10/1999
    
    starti=int(np.fix(wsi/2)) # Starting row for window processing
    startj=int(np.fix(wsj/2)) # Starting column for window processing
    
    stopi= int(nrows-inci)-1 # Stop row for window processing
    stopj= int(ncols-incj)-1 # Stop column for window processing
            
    for ii in np.arange(startj,stopj+1):

        # self.progress.emit(str(ii)+'/'+str(nrows))
        self.pBar.emit(int((ii/ncols)*90))
        for jj in np.arange(starti,stopi+1):
    
            t11s = np.nanmean(t11_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t12s = np.nanmean(t12_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t13s = np.nanmean(t13_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            
            t21s = np.nanmean(t21_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t22s = np.nanmean(t22_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t23s = np.nanmean(t23_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            
            t31s = np.nanmean(t31_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t32s = np.nanmean(t32_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t33s = np.nanmean(t33_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
    
            T_T1 = np.array([[t11s, t12s, t13s], [t21s, t22s, t23s], [t31s, t32s, t33s]])
    
            # %% RVI
            if np.isnan(np.real(T_T1)).any() or np.isinf(np.real(T_T1)).any() or np.isneginf(np.real(T_T1)).any():
                T_T1 = np.array([[0,0],[0,0]])
                temp_rvi[ii,jj] = 0
                # self.progress.emit(str('invalid Value encountered!!'))
                continue
                
            e_v = -np.sort(-np.linalg.eigvals(T_T1)); # sorting in descending order
            e_v1 = e_v[0]; e_v2 = e_v[1]; e_v3 = e_v[2];

            # self.progress.emit(str('Eigen val Done'))
            
            p1 = e_v1/(e_v1 + e_v2 + e_v3);
            p2 = e_v2/(e_v1 + e_v2 + e_v3);
            p3 = e_v3/(e_v1 + e_v2 + e_v3);
            
            p1=0 if p1<0 else p1
            p2=0 if p2<0 else p2
            p3=0 if p3<0 else p3
                
            p1=1 if p1>1 else p1
            p2=1 if p2>1 else p2
            p3=1 if p3>1 else p3
            
            
            
            temp_rvi[ii,jj] = np.real((4*p3)/(p1 + p2 + p3));
    
    # %% RVI scaled (0 - 1)   
    rvi = temp_rvi;   
    idx = np.argwhere(rvi>1)

    rvi[idx] = (3/4)*rvi[idx];
    rvi[~idx] = rvi[~idx];
    rvi[rvi==0] = np.NaN


    if write_flag:
        infile = T3_folder+'/T11.bin'
        """Write files to disk"""
        if os.path.exists(T3_folder+'/T11.bin'):
            infile = iFolder+'/T11.bin'
        elif os.path.exists(T3_folder+'/C11.bin'):
            infile = T3_folder+'/C11.bin'

        ofilegrvi = T3_folder+'/RVIFP.bin'
        write_bin(ofilegrvi,vi,infile)
                    

    return rvi



from osgeo import gdal
import numpy as np
import os 
import warnings
warnings.filterwarnings('ignore')

from .basic_func import read_bin, write_bin, conv2d


def grvi(T3_folder,window_size=1,write_flag=None):

    T11 = read_bin(T3_folder+"/T11.bin")
    T22 = read_bin(T3_folder+"/T22.bin")
    T33 = read_bin(T3_folder+"/T33.bin")

    T12_i = read_bin(T3_folder+'/T12_imag.bin')
    T12_r = read_bin(T3_folder+'/T12_real.bin')
    T13_i = read_bin(T3_folder+'/T13_imag.bin')
    T13_r = read_bin(T3_folder+'/T13_real.bin')
    T23_i = read_bin(T3_folder+'/T23_imag.bin')
    T23_r = read_bin(T3_folder+'/T23_real.bin')

    T12 = T12_r + 1j*T12_i
    T13 = T13_r + 1j*T13_i
    T23 = T23_r + 1j*T23_i
        
    t11_T1 = T11
    t12_T1 = T12
    t13_T1 = T13
    t21_T1 = np.conj(T12)
    t22_T1 = T22
    t23_T1 = T23
    t31_T1 = np.conj(T13)
    t32_T1 = np.conj(T23)
    t33_T1 = T33
    del T11, T22, T33,T12, T13, T23, T12_i,T12_r, T13_i, T13_r, T23_i, T23_r
                
    nrows  = np.shape(T11)[1]
    ncols = np.shape(T11)[0]
    # nrows  = 100
    # ncols = 100
   
    span = np.zeros((ncols,nrows))
    rho_13_hhvv = np.zeros((ncols,nrows))
    # temp_rvi = np.zeros((ncols,nrows))
    fp22 = np.zeros((ncols,nrows))
    GD_t1_t = np.zeros((ncols,nrows))
    GD_t1_d = np.zeros((ncols,nrows))
    GD_t1_rv = np.zeros((ncols,nrows))
    GD_t1_nd = np.zeros((ncols,nrows))
    GD_t1_c = np.zeros((ncols,nrows))
    GD_t1_lh = np.zeros((ncols,nrows))
    GD_t1_rh = np.zeros((ncols,nrows))
    beta = np.zeros((ncols,nrows))
    beta_1 = np.zeros((ncols,nrows))
    f = np.zeros((ncols,nrows))
    a = np.zeros((ncols,nrows))
    b = np.zeros((ncols,nrows))
    temp_gamma = np.zeros((ncols,nrows))
    t_d = np.zeros((46,1))
    t_nd = np.zeros((46,1))
    t_t = np.zeros((46,1))
    t_c = np.zeros((46,1))
    theta_map = np.zeros((ncols,nrows))
    
    D = (1/np.sqrt(2))*np.array([[1,0,1], [1,0,-1],[0,np.sqrt(2),0]])
    # %% for window processing
    wsi=wsj=window_size
    
    inci=int(np.fix(wsi/2)) # Up & down movement margin from the central row
    incj=int(np.fix(wsj/2)) # Left & right movement from the central column
    # % Starting row and column fixed by the size of the patch extracted from the image of 21/10/1999
    
    starti=int(np.fix(wsi/2)) # Starting row for window processing
    startj=int(np.fix(wsj/2)) # Starting column for window processing
    
    stopi= int(nrows-inci)-1 # Stop row for window processing
    stopj= int(ncols-incj)-1 # Stop column for window processing
            # %% Elementary targets
    
    M_d = np.array([[1,0,0,0], [ 0,1,0,0], [ 0,0,-1,0], [ 0,0,0,1]])
    M_nd = np.array([[0.625,0.375,0,0], [ 0.375,0.625,0,0], [ 0,0,-0.5,0], [ 0,0,0,0.5]])
    M_t = np.array([[1,0,0,0], [ 0,1,0,0], [ 0,0,1,0], [ 0, 0,0,-1]])
    M_c = np.array([[0.625,0.375,0,0], [ 0.375,0.625,0,0], [0,0,0.5,0], [ 0,0,0,-0.5]])
    M_lh = np.array([[1,0,0,-1], [ 0,0,0,0], [ 0,0,0,0], [ -1,0,0,1]])
    M_rh = np.array([[1,0,0,1], [ 0,0,0,0], [ 0,0,0,0], [ 1,0,0,1]])
    
    for ii in np.arange(startj,stopj+1):

        # self.progress.emit(str(ii)+'/'+str(nrows))
        self.pBar.emit(int((ii/ncols)*90))
        for jj in np.arange(starti,stopi+1):
    
            t11s = np.nanmean(t11_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t12s = np.nanmean(t12_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t13s = np.nanmean(t13_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            
            t21s = np.nanmean(t21_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t22s = np.nanmean(t22_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t23s = np.nanmean(t23_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            
            t31s = np.nanmean(t31_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t32s = np.nanmean(t32_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
            t33s = np.nanmean(t33_T1[ii-inci:ii+inci+1,jj-incj:jj+incj+1])#i sample
    
            T_T1 = np.array([[t11s, t12s, t13s], [t21s, t22s, t23s], [t31s, t32s, t33s]])
    
            #Coherency matrix
            C_T1 = np.matmul(np.matmul((D.T),T_T1),D);
            
            span[ii,jj] = np.real(t11s + t22s + t33s)
            temp_span = span[ii,jj]
            # self.progress.emit(str('span Done'))
            Temp_T1 = T_T1
            
            t11 = Temp_T1[0,0]; t12 = Temp_T1[0,1];        t13 = Temp_T1[0,2]
            t21 = np.conj(t12); t22 = Temp_T1[1,1];        t23 = Temp_T1[1,2]
            t31 = np.conj(t13); t32 = np.conj(t23);        t33 = Temp_T1[2,2]
            
            # %% Ratio of VV/HH (Used in Yamagichi Volume)
            
            hh = 0.5*(t11 + t22 + 2*np.real(t12)); #HH
            hv = t33; #HV
            vv = 0.5*(t11 + t22 - 2*np.real(t12)); #VV
            vol_con = 10*np.log10(vv/hh);
    
    
            # %% Kennaugh Matrix
            
            m11 = t11+t22+t33; m12 = t12+t21; m13 = t13+t31; m14 = -1j*(t23 - t32);
            m21 = t12+t21; m22 = t11+t22-t33; m23 = t23+t32; m24 = -1j*(t13-t31);
            m31 = t13+t31; m32 = t23+t32; m33 = t11-t22+t33; m34 = 1j*(t12-t21);
            m41 = -1j*(t23-t32); m42 = -1j*(t13-t31); m43 = 1j*(t12-t21); m44 = -t11+t22+t33;
            
            M_T = 0.5*np.array([[m11, m12, m13, m14], [m21, m22, m23, m24], [m31, m32, m33, m34], [m41, m42, m43, m44]]);
            
            
            M_T_theta = M_T;
            
            # %% GVSM
            
            t011 = M_T_theta[0,0] + M_T_theta[1,1] + M_T_theta[2,2] - M_T_theta[3,3];
            t012 = M_T_theta[0,1] - 1j*M_T_theta[2,3];
            t013 = M_T_theta[0,2] + 1j*M_T_theta[1,3];
            t021 = np.conj(t012);
            t022 = M_T_theta[0,0] + M_T_theta[1,1] - M_T_theta[2,2] + M_T_theta[3,3];
            t023 = M_T_theta[1,2] +1j*M_T_theta[0,3];
            t031 = np.conj(t013);
            t032 = np.conj(t023);
            t033 = M_T_theta[0,0] - M_T_theta[1,1] + M_T_theta[2,2] + M_T_theta[3,3];
            
            # %% T to C
            
            T0 = np.array([[t011/2, t012, t013], [t021, t022/2, t023], [t031, t032, t033/2]]);
            C0 = np.matmul(np.matmul((D.T),T0),D);
    
            # %% Gamma/Rho
            
            gamma = np.real(C0[0,0]/C0[2,2]); rho = 1/3;
            temp_gamma[ii,jj]= np.real(gamma); #% variable to save
            
            # %% Covariance matrix
            
            c11 = gamma; c12 = 0; c13 = rho*np.sqrt(gamma);
            c21 = 0; c22 = 0.5*(1 + gamma) - rho*np.sqrt(gamma); c23 = 0;
            c31 = np.conj(rho)*np.sqrt(gamma); c32 = 0; c33 = 1;
            
            R = (3/2)*(1 + gamma) - rho*np.sqrt(gamma);
            C1 = (1/R)*np.array([[c11, c12, c13], [c21, c22, c23], [c31, c32, c33]]);
            # self.progress.emit(str('gamma and R Done'))
            # %% Coherency matrix
            
            T1 = np.matmul(np.matmul(D,C1),(D.T));
            
            t11 = T1[0,0]; t12 = T1[0,1]; t13 = T1[0,2];
            t21 = T1[1,0]; t22 = T1[1,1]; t23 = T1[1,2];
            t31 = T1[2,0]; t32 = T1[2,1]; t33 = T1[2,2];
            
            m11 = t11+t22+t33; m12 = t12+t21; m13 = t13+t31; m14 = -1j*(t23 - t32);
            m21 = t12+t21; m22 = t11+t22-t33; m23 = t23+t32; m24 = -1j*(t13-t31);
            m31 = t13+t31; m32 = t23+t32; m33 = t11-t22+t33; m34 = 1j*(t12-t21);
            m41 = -1j*(t23-t32); m42 = -1j*(t13-t31); m43 = 1j*(t12-t21); m44 = -t11+t22+t33;
            
            # %% Generalized Random Volume (Antropov et al.)
            
            M_rv = np.real(np.array([[m11, m12, m13, m14], [m21, m22, m23, m24], [m31, m32, m33, m34], [m41, m42, m43, m44]]));
            
            f[ii,jj] = 1;
    
            # %% GD Volume
            
            num1 = np.matmul(((M_T_theta).T),M_rv); #% volume
            num = np.trace(num1);
            den1 = np.sqrt(abs(np.trace(np.matmul(((M_T_theta).T),M_T_theta))));
            den2 = np.sqrt(abs(np.trace(np.matmul(((M_rv).T),M_rv))));
            den = den1*den2;
            temp_aa = np.real(2*np.arccos(num/den)*180/np.pi);
            GD_t1_rv[ii,jj] = np.real(temp_aa/180);
            # self.progress.emit(str('GD volume Done'))
            # %% GD ALL
            
            num1 = np.matmul(((M_T_theta).T),M_c); #% cylinder
            num = np.trace(num1);
            den1 = np.sqrt(abs(np.trace(np.matmul(((M_T_theta).T),M_T_theta))));
            den2 = np.sqrt(abs(np.trace(np.matmul(((M_c).T),M_c))));
            den = den1*den2;
            temp_aa = np.real(2*np.arccos(num/den)*180/np.pi);
            GD_t1_c[ii,jj] = np.real(temp_aa/180);
            # self.progress.emit(str('GD cylider Done'))
            
            num1 = np.matmul(((M_T_theta).T),M_t); #% trihedral
            num = np.trace(num1);
            den1 = np.sqrt(abs(np.trace(np.matmul(((M_T_theta).T),M_T_theta))));
            den2 = np.sqrt(abs(np.trace(np.matmul(((M_t).T),M_t))));
            den = den1*den2;
            temp_aa = 2*np.arccos(num/den)*180/np.pi;
            GD_t1_t[ii,jj] = np.real(temp_aa/180);
            # self.progress.emit(str('GD trihedral Done'))
            
            num1 = np.matmul(((M_T_theta).T),M_d); #% dihedral
            num = np.trace(num1);
            den1 = np.sqrt(abs(np.trace(np.matmul(((M_T_theta).T),M_T_theta))));
            den2 = np.sqrt(abs(np.trace(np.matmul(((M_d).T),M_d))));
            den = den1*den2;
            temp_aa = 2*np.arccos(num/den)*180/np.pi;
            GD_t1_d[ii,jj] = np.real(temp_aa/180);
            # self.progress.emit(str('GD dihedral Done'))
            
            num1 = np.matmul(((M_T_theta).T),M_nd); #% n-dihedral
            num = np.trace(num1);
            den1 = np.sqrt(abs(np.trace(np.matmul(((M_T_theta).T),M_T_theta))));
            den2 = np.sqrt(abs(np.trace(np.matmul(((M_nd).T),M_nd))));
            den = den1*den2;
            temp_aa = 2*np.arccos(num/den)*180/np.pi;
            GD_t1_nd[ii,jj] = np.real(temp_aa/180);
            # self.progress.emit(str('GD n-dihedral Done'))
            
            # %% VI
            
            t_t = GD_t1_t[ii,jj];
            t_d = GD_t1_d[ii,jj];
            t_c = GD_t1_c[ii,jj];
            t_nd = GD_t1_nd[ii,jj];
            
            a[ii,jj] = np.nanmax([t_t, t_d, t_c, t_nd]);
            b[ii,jj] = np.nanmin([t_t, t_d, t_c, t_nd]);
            beta[ii,jj] = (b[ii,jj]/a[ii,jj])**2;
            
    # %% GRVI
    f[f==0]=np.NaN
    vi = np.power(beta, GD_t1_rv)*(1 - (1/f)*GD_t1_rv);
    
    x1 = np.power(beta, GD_t1_rv);
    x2 = (1 - GD_t1_rv);
    
    f =  np.nan_to_num(f)
    idx1 = np.argwhere(GD_t1_rv>f)
    vi[idx1] = 0;
    vi[~idx1] = vi[~idx1];
    

    if write_flag:
        infile = T3_folder+'/T11.bin'
        """Write files to disk"""
        if os.path.exists(T3_folder+'/T11.bin'):
            infile = iFolder+'/T11.bin'
        elif os.path.exists(T3_folder+'/C11.bin'):
            infile = T3_folder+'/C11.bin'

        ofilegrvi = T3_folder+'/GRVI.bin'
        write_bin(ofilegrvi,vi,infile)
                    

    return vi



from setuptools import setup

setup(
    name='polsartools',
    version='0.1',    
    description='A package to process Synthetic Aperture Radar data',
    url='https://github.com/Narayana-Rao/polsartools',
    author='Narayanarao Bhogapurapu',
    author_email='bnarayanarao@iitb.ac.in',
    license='GPL-3.0 license',
    packages=['polsartools'],
    install_requires=['gdal',
                      'numpy',                     
                      ],

    # classifiers=[
    #     'Development Status :: 1 - alpha',
    #     'Intended Audience :: Science/Research',
    #     'License :: OSI Approved ::  GPL-3.0 license',  
    #     'Operating System :: POSIX :: Linux',        
    #     'Programming Language :: Python :: 3',
    # ],
)


"""
polsartools.

A package to process Synthetic Aperture Radar data
"""
from .mf3cf import mf3cf
from .mf4cf import mf4cf
from .dopfp import dopfp
from .convert import T3_C3, C3_compact_C2, C3_c2, S2_T3


############################
__version__ = "0.1"
__author__ = 'Narayanarao Bhogapurapu'
__credits__ = 'Microwave Remote Sensing Lab (MRSLab)'


from osgeo import gdal
import numpy as np

def conv2d(a, f):
    filt = np.zeros(a.shape)
    wspad = int(f.shape[0]/2)
    s = f.shape + tuple(np.subtract(a.shape, f.shape) + 1)
    strd = np.lib.stride_tricks.as_strided
    subM = strd(a, shape = s, strides = a.strides * 2)
    filt_data = np.einsum('ij,ijkl->kl', f, subM)
    filt[wspad:wspad+filt_data.shape[0],wspad:wspad+filt_data.shape[1]] = filt_data
    return filt

def read_bin(file):
    ds = gdal.Open(file)
    band = ds.GetRasterBand(1)
    arr = band.ReadAsArray()
    return arr

def write_bin(file,wdata,refData):        
    ds = gdal.Open(refData)
    [cols, rows] = wdata.shape   
    driver = gdal.GetDriverByName("ENVI")
    outdata = driver.Create(file, rows, cols, 1, gdal.GDT_Float32)
    outdata.SetGeoTransform(ds.GetGeoTransform())
    outdata.SetProjection(ds.GetProjection())   
    outdata.SetDescription(file)
    outdata.GetRasterBand(1).WriteArray(wdata)
    outdata.FlushCache() 